package org.logicblaze.ldap;

/**
 *
 * Copyright 2005-2006 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.liferay.portal.service.UserManagement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.naming.NamingException;

public class ApacheDSUserManagement implements UserManagement {

    private static final Log log = LogFactory.getLog(ApacheDSUserManagement.class);

    public int addUser(String userId, String password, String emailAddress, String firstName, String lastName) {

        try {
            UserServiceUtil.addUser(userId, password, emailAddress, firstName, lastName);

        } catch (NamingException ne) {
            log.error("Error Adding User: " + ne.getMessage());
            return FAILURE;

        }

        return SUCCESS;
    }

    public int deleteUser(String userId) {

        try {
            UserServiceUtil.deleteUser(userId);
        } catch (NamingException ne) {
            log.error("Error Deleting User: " + ne.getMessage());
            return FAILURE;
        }

        return SUCCESS;
    }

    public int updateUser(String userId, String emailAddress, String firstName, String lastName) {

        try {
            UserServiceUtil.updateUser(userId, emailAddress, firstName, lastName);
        } catch (NamingException ne) {
            log.error("Error Updating User: " + ne.getMessage());
            return FAILURE;
        }

        return SUCCESS;
    }

    public int updatePassword(String userId, String password) {

        try {
            UserServiceUtil.updatePassword(userId, password);
        } catch (NamingException ne) {
            log.error("Error Updating Password: " + ne.getMessage());
            return FAILURE;
        }

        return SUCCESS;
    }


    public int addRoleUsers(String roleName, String[] userIds) {

        try {
            UserServiceUtil.addRoleUsers(roleName, userIds);

        } catch (NamingException ne) {
            log.error("Error Adding Users to Role: " + ne.getMessage());
            return FAILURE;

        }

        return SUCCESS;
    }

    public int unsetRoleUsers(String roleName, String[] userIds) {

        try {
            UserServiceUtil.unsetRoleUsers(roleName, userIds);

        } catch (NamingException ne) {
            log.error("Error Removing Users to Role: " + ne.getMessage());
            return FAILURE;

        }

        return SUCCESS;
    }

    public int addRole(String roleName) {

        try {
            UserServiceUtil.addRole(roleName);

        } catch (NamingException ne) {
            log.error("Error Creating Role: " + ne.getMessage());
            return FAILURE;

        }

        return SUCCESS;
    }

}
