/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/

package org.logicblaze.ldap;

import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;

public class EntityDirContext extends AbstractDirContext {

	String type;

	Attributes myAttrs;

	public EntityDirContext(String dn, String userId,	String password, String email,
                          String firstname, String lastname) {

		this.type = dn;

		myAttrs = new BasicAttributes(true);

		Attribute oc = new BasicAttribute("objectclass");

		oc.add("top");
		oc.add("person");
		oc.add("organizationalPerson");
		oc.add("inetOrgPerson");

		myAttrs.put(oc);
		myAttrs.put("uid", userId);
		myAttrs.put("givenName", firstname);
        myAttrs.put("userPassword", password);
		myAttrs.put("sn", lastname);
		myAttrs.put("cn", firstname + " " + lastname);
		myAttrs.put("mail", email);
	}

    public EntityDirContext(String dn, String userId,	String email,
                          String firstname, String lastname) {

		this.type = dn;

		myAttrs = new BasicAttributes(true);

		Attribute oc = new BasicAttribute("objectclass");

		oc.add("top");
		oc.add("person");
		oc.add("organizationalPerson");
		oc.add("inetOrgPerson");

		myAttrs.put(oc);
		myAttrs.put("uid", userId);
		myAttrs.put("givenName", firstname);
		myAttrs.put("sn", lastname);
		myAttrs.put("cn", firstname + " " + lastname);
		myAttrs.put("mail", email);
	}

    public EntityDirContext(String dn, String userId, String password) {

		this.type = dn;

		myAttrs = new BasicAttributes(true);

		Attribute oc = new BasicAttribute("objectclass");

		oc.add("top");
		oc.add("person");
		oc.add("organizationalPerson");
		oc.add("inetOrgPerson");

		myAttrs.put(oc);
		myAttrs.put("uid", userId);
		myAttrs.put("userPassword", password);
	}

    /**
     * Used to add new roles.
     * @param dn
     * @param roleId
     */
    public EntityDirContext(String dn, String roleId){
        this.type = dn;

        myAttrs = new BasicAttributes(true);

        Attribute oc = new BasicAttribute("objectclass");

        oc.add("top");
        oc.add("groupOfUniqueNames");

        myAttrs.put(oc);
        myAttrs.put("cn", roleId);
    }

	public Attributes getAttributes(String name) throws NamingException {
		if (!name.equals("")) {
			throw new NameNotFoundException();
		}
		return (Attributes) myAttrs.clone();
	}

	public String toString() {
		return type;
	}
}
