/**
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.logicblaze.soa;

import java.io.File;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

import net.sf.retrotranslator.transformer.JITRetrotranslator;

import org.apache.geronimo.gbean.GBeanInfo;
import org.apache.geronimo.gbean.GBeanInfoBuilder;
import org.apache.geronimo.gbean.GBeanLifecycle;

/**
 * @version $Revision: 66 $
 */
public class BootstrapGBean implements GBeanLifecycle {

    Main app = new Main();
    String args[] = new String[]{};
    private boolean useDefExt=true;
    
    public void doStart() throws Exception {

       

        // Enable retrotranslator 
        JITRetrotranslator.install();
        app.setMainClassName("org.logicblaze.soa.GeronimoBootstrapMain");        
        app.setParentClassLoader(ClassLoader.getSystemClassLoader());
        
        // Convert arguments to collection for easier management
        List tokens = new LinkedList(Arrays.asList(args));
        // Parse for extension directory option
        app.parseExtensions(tokens);

        // Add default extension directories
        /*
         * if(useDefExt && app.canUseExtdir()) {
         */
        if (useDefExt && app.canUseExtdir()) {
            File shared = new File(app.getHome(), "shared");
            app.addExtensionDirectory(new File(shared, "lib"));
            app.addExtensionDirectory(new File(shared, "classes"));
        } else {
            if( app.isVerbose() ) {
                System.out.println("Extension directories were not added");
            }
        }

        try {
            app.main(tokens);
        }
        catch (Throwable e) {
            System.out.println("Failed to execute main task. Reason: " + e);
            if( app.isVerbose() ) {
                e.printStackTrace();
            }
        }
        
        
    }

    public void doStop() throws Exception {
        app.main(Arrays.asList(new String[]{"stop"}));
    }

    public void doFail() {
        try {
            app.main(Arrays.asList(new String[]{"stop"}));
        } catch (Exception e) {
        }
    }

    public File getHome() {
        return app.getHome();
    }

    public void setHome(File home) {
    	
  	

       if (!home.isDirectory() || home.getName().equalsIgnoreCase("bin")) {
            if (System.getProperty("fuse.home") != null) {
                 home =  new File(System.getProperty("fuse.home"));
            } else { 
                URL url = Main.class.getClassLoader().getResource("org/logicblaze/soa/Main.class");
                if (url != null) {
                    try {
                        JarURLConnection jarConnection = (JarURLConnection) url.openConnection();
                        url = jarConnection.getJarFileURL();
                        URI baseURI = new URI(url.toString()).resolve(".");
                        String  homePath = new File(baseURI).getCanonicalFile().getAbsolutePath() + File.separator + ".." + File.separator + ".." + File.separator + ".." + File.separator + "fuse" ;
                        home = new File (homePath).getCanonicalFile();

                    }
                    catch (Exception ignored) {
                    }
                }	
                 
           
            }
            
           
        }
    	
        app.setHome(home);
    }

    public String[] getArgs() {
        return args;
    }

    public void setArgs(String[] args) {
        this.args = args;
    }

    public boolean isUseDefExt() {
        return useDefExt;
    }

    public void setUseDefExt(boolean useDefExt) {
        this.useDefExt = useDefExt;
    }
    
    public static final GBeanInfo GBEAN_INFO;

    static {
        GBeanInfoBuilder infoFactory = new GBeanInfoBuilder("Fuse Bootstrap", BootstrapGBean.class, "Fuse");
        infoFactory.addAttribute("useDefExt", boolean.class, true);
        infoFactory.addAttribute("verbose", boolean.class, true);
        infoFactory.addAttribute("args", String[].class, true);
        infoFactory.addAttribute("home", File.class, true);
        GBEAN_INFO = infoFactory.getBeanInfo();
    }

    public static GBeanInfo getGBeanInfo() {
        return GBEAN_INFO;
    }

    public void addExtensionDirectory(File directory) {
        app.addExtensionDirectory(directory);
    }

    public boolean isVerbose() {
        return app.isVerbose();
    }

    public void setVerbose(boolean verbose) {
        app.setVerbose(verbose);
    }

}
